/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * private.h
 * Private definitions, data structures, macros and functions for libsandnet
 * Copyright (C) 2005 Simon Newton
 */

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sandnet.h> 
#include <sandnet/packets.h> 

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>


#ifndef _defs_defined
#define _defs_defined


#define SANDNET_CPORT 37895		// dport for control msgs
#define SANDNET_DPORT 37900		// dport for data

#define SANDNET_FIRMWARE_VER	0x00050501		// firmware version
#define SANDNET_CTL_ADDR		"237.1.1.1"		// control data multicast address
#define SANDNET_DATA_ADDR		"237.1.2.1"		// dmx multicast address

#define RECV_NO_DATA 1
#define REPEAT_FLAG 0x80


/*
enum sandnet_port_proto_e {


}__attribute__( ( packed ) );

typedef
*/


// non sandnet specific
#define TRUE 1
#define FALSE 0

#define SA struct sockaddr
#define SI struct in_addr

#define min(a,b)    ((a) < (b) ? (a) : (b))
#define max(a,b)    ((a) > (b) ? (a) : (b))

// byte ordering macros
#define bswap_16(x)	((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))

// htols : convert short from host to little endian order
// ltohs : convert short from little endian to host order
#ifdef HAVE_ENDIAN_H
# if BYTE_ORDER == __BIG_ENDIAN
#  define htols(x) 	bswap_16 (x)
# else
#  define htols(x)	(x)
# endif
#else
# if BYTE_ORDER == BIG_ENDIAN
#  define htols(x) 	bswap_16 (x)
# else
#  define htols(x)	(x)
# endif
#endif

#define short_gethi(x) ((0xff00 & x) >> 8)
#define short_getlo(x)  (0x00ff & x)


#define sandnet_error_malloc() sandnet_error("%s : malloc error %s" , __FUNCTION__, strerror(errno) ) 
#define sandnet_error_realloc() sandnet_error("%s : realloc error %s" , __FUNCTION__, strerror(errno) ) 
#define sandnet_error_nullnode() sandnet_error("%s : argument 1 (sandnet_node) was null" , __FUNCTION__ ) 
										
// check if the node is null and return an error
#define return_if_null(node) if (node == NULL) { \
	sandnet_error_nullnode() ; \
	return SANDNET_EARG ; \
}

// return if the node is off
#define return_if_off(node) if (node->state.mode == SANDNET_OFF) { \
	sandnet_error("%s : node is not on" , __FUNCTION__ ) ; \
	return SANDNET_ESTATE ; \
}
										
// return if the node is on
#define return_if_on(node) if (node->state.mode == SANDNET_ON) { \
	sandnet_error("%s : node is not on" , __FUNCTION__ ) ; \
	return SANDNET_ESTATE ; \
}



/*
 * Data structures to manage callbacks
 */

// packet callbacks have a function and some user data
typedef struct {
	int (*fh)(sandnet_node n, void *p, void *data) ;
	void *data;	
} callback_t ;


/*
 * the dmx callback is triggered when a dmx packet arrives
 */
typedef struct {
	int (*fh)(sandnet_node n, uint8_t group, uint8_t universe, int length, uint8_t *data, void *d) ;
	void *data;	
} dmx_callback_t ;



// the status of the node
typedef enum {
	SANDNET_OFF,
	SANDNET_ON
} node_status_t ;

// struct to hold the state of the node
typedef struct {
	uint8_t hw_addr[6] ;
	uint16_t packet_count ;
	int verbose ;
	SI ip_addr;
	SI ctl_addr ;
	SI data_addr;
	char name[SANDNET_NAME_LENGTH + 1] ;		// null terminated
	node_status_t mode;
	uint8_t		led;						// led state
} node_state_t ;



/*
 * The port state
 */
typedef struct {
	uint8_t		proto;
	uint8_t		mode;
	uint8_t		term;
	uint8_t		b;
	uint8_t		param1;
	uint8_t		param2;
} node_port_t ;


/**
 * The main node structure
 */
typedef struct sandnet_node_s{
	int sd[2] ;											// the two sockets (ctl and data)
	node_state_t state;									// the state struct
	node_port_t	ports[SANDNET_MAX_PORTS];				// 2 ports per node
	dmx_callback_t dmx_callback;						// dmx callback
	sandnet_ad_t		ad_temp;						// buffered advertisment
	callback_t		callbacks[SANDNET_LAST_HANDLER] ;	// callbacks
} sandnet_node_t ;

							
typedef sandnet_node_t *node ;


// Global variables
extern char *sandnet_errstr ;


// Function definitions follow
 
// sandnet.c
void sandnet_error(const char *fmt, ...) ;


//transmit.c
int sandnet_tx_build_ad(node n);
int sandnet_tx_send_ad(node n);
int sandnet_tx_encode_dmx(uint8_t *src, int slen, uint8_t *dst, int dlen);
int sandnet_tx_send_dmx(node n, uint8_t port, uint8_t grp, uint8_t universe, int length, uint8_t *data);

//receive.c
int sandnet_rx_handle_ctl(node n, sandnet_packet p) ;
int sandnet_rx_handle_data(node n, sandnet_packet p) ;


// exported from network.c
ssize_t sandnet_net_recv(node n, sandnet_packet p, int block) ;
int sandnet_net_ctl_send(node n, sandnet_packet p) ;
int sandnet_net_data_send(node n, sandnet_packet p) ;
int sandnet_net_init(node n, const char *ip) ;
int sandnet_net_start(node n) ;
int sandnet_net_close(node n) ;


#endif
