/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * receive.c
 * Functions to handle sending datagrams
 * Copyright (C) 2006 Simon Newton
 */

#include "private.h"



/** checks if the callback is defined, if so call it passing the packet and 
 * the user supplied data. 
 * If the callbacks return a non-zero result, further processing is canceled.
 */
int check_callback(node n, sandnet_packet p, callback_t callback) {
	if(callback.fh != NULL) 
		return callback.fh(n,p, callback.data) ;

	return 0;
}




/*
 * Returns true if the packet for directed at us (the mac address matches)
 */
static int sandnet_rx_for_us(node n, sandnet_packet p) {

	if( p->length >= 8 && (memcmp(p->data.ad.mac, n->state.hw_addr, SANDNET_MAC_LENGTH) == 0) ) 
		return 1 ;

	return 0;
}


/*
 * Handle a port control packet
 */
static int sandnet_rx_handle_pctl(node n, sandnet_packet p) {
	int i = 0 ;

	// run callback if defined
	if(check_callback(n, p, n->callbacks[SANDNET_CTL_HANDLER]) )
		return SANDNET_EOK;

	for(i=0 ; i < SANDNET_MAX_PORTS; i++) {
		n->ports[i].mode = p->data.pctl.ports[i].mode ;
		n->ports[i].proto = p->data.pctl.ports[i].proto ;
		n->ports[i].term = p->data.pctl.ports[i].term ;
		n->ports[i].b = p->data.pctl.ports[i].b ;
		n->ports[i].param1 = p->data.pctl.ports[i].param1 ;
		n->ports[i].param2 = p->data.pctl.ports[i].param2 ;

		printf("%hhx %hhx %hhx %hhx %hhx %hhx\n" , n->ports[i].mode, n->ports[i].proto, n->ports[i].term,
				n->ports[i].b, n->ports[i].param1, n->ports[i].param2) ;
	}

	//build an ad and send
	sandnet_tx_build_ad(n) ;
	sandnet_tx_send_ad(n);
	return SANDNET_EOK;
}


/*
 * Handles a set name packet
 */
static int sandnet_rx_handle_name(node n, sandnet_packet p) {

	// run callback if defined
	if(check_callback(n, p, n->callbacks[SANDNET_NAME_HANDLER]) )
		return SANDNET_EOK;

	int len = min(p->data.name.nlen, SANDNET_NAME_LENGTH) ;
	memset(n->state.name, 0x00, SANDNET_NAME_LENGTH) ;
	strncpy(n->state.name , (char *) p->data.name.name, len ) ;

	// build an ad and send
	sandnet_tx_build_ad(n) ;
	sandnet_tx_send_ad(n);	
	return SANDNET_EOK ;
}


/*
 * Handles an identify packet
 */
static int sandnet_rx_handle_identify(node n, sandnet_packet p) {

	// run callback if defined
	if(check_callback(n, p, n->callbacks[SANDNET_IDENTIFY_HANDLER]) )
		return SANDNET_EOK;

	return SANDNET_EOK ;
}


/*
 * Handles a set led packet
 */
static int sandnet_rx_handle_led(node n, sandnet_packet p) {

	// run callback if defined
	if(check_callback(n, p, n->callbacks[SANDNET_LED_HANDLER]) )
		return SANDNET_EOK;

	n->state.led = p->data.led.led ;

	//build an ad and send
	sandnet_tx_build_ad(n) ;
	sandnet_tx_send_ad(n);	
	return SANDNET_EOK ;
}


/**
 * handle a sandnet dmx packet
 *
 */
static int sandnet_rx_handle_dmx(node n, sandnet_packet p) {
	int len, enc_len, start, uni ;
	uint8_t dmx[SANDNET_DMX_LENGTH] ;

	memset(dmx, 0x00, SANDNET_DMX_LENGTH) ;
	if(n->dmx_callback.fh == NULL) 
		return 0 ;

	// TODO: this is a mega hack
	// fix it when we have access to the strand again
	// hmm seems to work... :)
//	enc_len = p->data.dmx.indexBlock[1] - p->data.dmx.indexBlock[0] ;
//	uni = p->data.dmx.netSlot[0] / SANDNET_DMX_LENGTH ;

//	start = (p->data.dmx.netSlot[0] % SANDNET_DMX_LENGTH) - 1;
	
//	len = decode_dmx(p->data.dmx.data, enc_len, &dmx[start], sizeof(dmx)-start) ;

//	n->dmx_callback.fh(n, uni, len, dmx, n->dmx_callback.data) ;

	return 0;
}


/*
 * decode sandnet data
 */
static int sandnet_rx_decode_dmx(uint8_t *src, int slen, uint8_t *dst, int dlen) {
	int i,l ;
	int lr = dlen ;
	int di = 0;

	for(i=0; i < slen && lr > 0 ; ) {
			
		l = src[i] & (~REPEAT_FLAG) ;

		l = min(lr, l) ;

		if(src[i] & REPEAT_FLAG) {
			i++ ;
			memset(&dst[di], src[i++], min(l, lr) ) ;
			di += l ;
		} else {
			memcpy(&dst[di], &src[++i], min(l, lr) ) ;
			di += l;
			i += l ;
		}
		lr -= l ;
	}
	return di ;
}


/**
 * the main handler for an sandnet packet calls 
 * the appropriate handler function
 *
 */
int sandnet_rx_handle_ctl(node n, sandnet_packet p) {
	uint16_t	op ;
	int mine = 0 ;

	if ( p->length < 2 )
		return SANDNET_EOK;
	
	op = ntohs(p->data.ad.op) ;
	mine = sandnet_rx_for_us(n,p) ;

	if(check_callback(n, p, n->callbacks[SANDNET_RECV_HANDLER]) )
		return 0;

	if ( op == SANDNET_AD && p->length >= sizeof(sandnet_ad_t) ) {
		// sandnet advertisment
		// we don't really care about these

	} else if ( op == SANDNET_CTL && mine && p->length >= sizeof(sandnet_pctl_t) ) {
		// port control
		sandnet_rx_handle_pctl(n,p) ;

	} else if ( op == SANDNET_NAME && mine && p->length >= sizeof(sandnet_name_t) ) {
		// set name
		sandnet_rx_handle_name(n, p) ;

	} else if ( op == SANDNET_IDENTIFY && mine && p->length >= sizeof(sandnet_id_t) ) {
		// identify packet
		sandnet_rx_handle_identify(n,p) ;

	} else if ( op == SANDNET_PROG && mine && p->length >= sizeof(sandnet_prog_t) ) {
		// program, don't care about this


	} else if ( op == SANDNET_LED && mine && p->length >= sizeof(sandnet_led_t) ) {
		// change led
		sandnet_rx_handle_led(n,p) ;
	}

	return SANDNET_EOK ;
}


/*
 * handle data packets
 */
int sandnet_rx_handle_data(node n, sandnet_packet p) {
	uint16_t	op ;

	if ( p->length < 2 )
		return SANDNET_EOK;
	
	op = ntohs(p->data.ad.op) ;

	if(check_callback(n, p, n->callbacks[SANDNET_RECV_HANDLER]) )
		return 0;

	if(op == SANDNET_DMX && p->length >= sizeof(sandnet_dmx_t) ) {
		if(n->dmx_callback.fh != NULL) {
			n->dmx_callback.fh(n, p->data.dmx.grp , p->data.dmx.uni , SANDNET_DMX_LENGTH , p->data.dmx.dmx, n->dmx_callback.data) ;
		}
	} else if (op == SANDNET_DMX2 && p->length >= sizeof(sandnet_data_t) ) {
		// the elusive encoded data packet
		printf("got encoded data\n") ;

	}
	return SANDNET_EOK ;

}
